package solatium.blockchain;

import java.util.*;

public final class Chain {

    private final String GENESIS_HASH = "0";
    private final Deque<HashedBlock> blockChain = new ArrayDeque<>();

    public Chain() {
        Block genesisBlock = new TransactionBlock(GENESIS_HASH, "Genesis", 0);
        HashedBlock hashedGenesisBlock = genesisBlock.getHashed();
        blockChain.addLast(hashedGenesisBlock);
    }

    public void addToChain(Block block) {
        blockChain.addLast(block.getHashed());
    }

    public void addToChain(HashedBlock block) {
        blockChain.addLast(block);
    }

    public void addToChain(String data) {
        Block block = new TransactionBlock(getLastHash(), data, new Date().getTime());
        addToChain(block.getHashed());
    }

    public boolean validateChain() {
        Iterator<HashedBlock> hashedBlockIterator = blockChain.iterator();
        String previousHash = GENESIS_HASH;

        while (hashedBlockIterator.hasNext()) {
            HashedBlock hashedBlock = hashedBlockIterator.next();
            String originalHash = hashedBlock.getHash();
            Block block = hashedBlock.getBlock();
            String newHash = new Sha256Hash(block.getDataToHash(), previousHash).getHash();
            if (!originalHash.equals(newHash) || !block.getPreviousHash().equals(previousHash)) {
                return false;
            }
            previousHash = originalHash;
        }

        return true;
    }

    public String getLastHash() {
        return blockChain.getLast().getHash();
    }

    public List<HashedBlock> getBlockChain() {
        List<HashedBlock> blockchainList = new ArrayList<>();

        blockChain.iterator().forEachRemaining(blockchainList::add);
        return blockchainList;
    }

    @Override
    public String toString() {
        List<HashedBlock> hashedBlocks = getBlockChain();
        StringBuilder hashedBlocksString = new StringBuilder();
        hashedBlocks.forEach(hashedBlock -> hashedBlocksString.append(hashedBlock.toString() + "\n"));
        return "Chain{\n" +
                hashedBlocksString.toString() +
                '}';

    }
}
