package solatium.blockchain;

import org.junit.Test;

import java.util.Date;
import java.util.List;

import static org.junit.Assert.*;

public class ChainTest {

    @Test
    public void shouldAddToChain() {
        String hash = "foo123";
        Block block = new TransactionBlock("", "", new Date().getTime());
        HashedBlock hashedBlock = new HashedBlock(block, hash);
        Chain chain = new Chain();
        chain.addToChain(hashedBlock);
        assertEquals(hash, chain.getLastHash());
    }

    @Test
    public void shouldValidateChain() {
        Chain chain = new Chain();
        for (int i = 0; i < 10; i++) {
            String previousHash = chain.getLastHash();
            chain.addToChain(getHashedBlock(previousHash));
        }
        assertTrue(chain.validateChain());
    }

    @Test
    public void shouldFailValidateChain() {
        Block wrongBlock = new TransactionBlock("", "", new Date().getTime());
        Chain chain = new Chain();
        for (int i = 0; i < 10; i++) {
            String previousHash = chain.getLastHash();
            chain.addToChain(getHashedBlock(previousHash));
        }
        chain.addToChain(wrongBlock.getHashed());
        assertFalse(chain.validateChain());
    }

    @Test
    public void shouldReturnListWithBlocks() {
        Chain chain = new Chain();
        Block block = new TransactionBlock(chain.getLastHash(), "test", new Date().getTime());
        chain.addToChain(block.getHashed());
        List<HashedBlock> hashedBlocks = chain.getBlockChain();
        assertEquals(2, hashedBlocks.size());
        assertEquals("Genesis", hashedBlocks.get(0).getBlock().getData());
        assertEquals("test", hashedBlocks.get(1).getBlock().getData());
    }

    @Test
    public void shouldToString() {
        Chain chain = new Chain();
        Block block0 = new TransactionBlock(chain.getLastHash(), "test", 1);
        Block block1 = new TransactionBlock(chain.getLastHash(), "test", 2);
        Block block2 = new TransactionBlock(chain.getLastHash(), "test", 3);
        chain.addToChain(block0.getHashed());
        chain.addToChain(block1.getHashed());
        chain.addToChain(block2.getHashed());
        assertEquals("Chain{\n" +
                "HashedBlock{hash='9A73FFC9E59030D6EA56CA972411E97A3BE8EF0E6D355AF80EAFA3D1B1AD366A', block=Block{previousHash='0', data='Genesis', timestamp=0}}\n" +
                "HashedBlock{hash='3985564A740D2B2BD098B3E0AD833C390B95DBB13A35BFA3E517747624761AA9', block=Block{previousHash='9A73FFC9E59030D6EA56CA972411E97A3BE8EF0E6D355AF80EAFA3D1B1AD366A', data='test', timestamp=1}}\n" +
                "HashedBlock{hash='2A03E73DD5BA364E0A57D33E8FC332305E0E52D085208B2C31FC695A36F74D86', block=Block{previousHash='9A73FFC9E59030D6EA56CA972411E97A3BE8EF0E6D355AF80EAFA3D1B1AD366A', data='test', timestamp=2}}\n" +
                "HashedBlock{hash='ED7C9D0AAA4F787AD3CD2582F867FA2763EA4268D49CF76C3D4DF4CF65803ACA', block=Block{previousHash='9A73FFC9E59030D6EA56CA972411E97A3BE8EF0E6D355AF80EAFA3D1B1AD366A', data='test', timestamp=3}}\n" +
                "}", chain.toString());

    }

    private HashedBlock getHashedBlock(String previousHash) {
        Block block = new TransactionBlock(previousHash, "Some basic Data to hash", new Date().getTime());
        return block.getHashed();
    }
}
